@@ -1,6 +1,6 @@
 #!/usr/bin/env python2
 """
-Copyright (c) 2013-2015 Ben Croston
+Copyright (c) 2013-2016 Ben Croston
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
@@ -67,8 +67,12 @@ class TestAAASetup(unittest.TestCase):
         GPIO.setup(LED_PIN_BCM, GPIO.IN)
         GPIO.cleanup()
 
+        # Test setting an invalid mode
+        with self.assertRaises(ValueError):
+            GPIO.setmode(666)
+
         # Test getmode()
-        self.assertEqual(GPIO.getmode(), GPIO.UNKNOWN)
+        self.assertEqual(GPIO.getmode(), None)
         GPIO.setmode(GPIO.BCM)
         self.assertEqual(GPIO.getmode(), GPIO.BCM)
         GPIO.setup(LED_PIN_BCM, GPIO.IN)
@@ -97,10 +101,9 @@ class TestAAASetup(unittest.TestCase):
         GPIO.setmode(GPIO.BOARD)
         with open('/sys/class/gpio/export','wb') as f:
             f.write(str(LED_PIN_BCM).encode())
+        time.sleep(0.2)  # wait for udev to set permissions
         with open('/sys/class/gpio/gpio%s/direction'%LED_PIN_BCM,'wb') as f:
             f.write(b'out')
-        with open('/sys/class/gpio/gpio%s/value'%LED_PIN_BCM,'wb') as f:
-            f.write(b'1')
         time.sleep(0.2)
         with warnings.catch_warnings(record=True) as w:
             GPIO.setup(LED_PIN, GPIO.OUT)    # generate 'already in use' warning
@@ -161,6 +164,16 @@ class TestAAASetup(unittest.TestCase):
         with self.assertRaises(ValueError):
             GPIO.setup(('d',LED_PIN), GPIO.OUT)
 
+        # test setting pull_up_down on an output
+        GPIO.setmode(GPIO.BOARD)
+        with self.assertRaises(ValueError):
+            GPIO.setup(LOOP_OUT, GPIO.OUT, pull_up_down=GPIO.PUD_DOWN)
+
+        # test setting initial on an input
+        GPIO.setmode(GPIO.BOARD)
+        with self.assertRaises(ValueError):
+            GPIO.setup(LOOP_IN, GPIO.IN, initial=GPIO.LOW)
+
 class TestInputOutput(unittest.TestCase):
     def setUp(self):
         GPIO.setmode(GPIO.BOARD)
@@ -265,10 +278,9 @@ class TestSetWarnings(unittest.TestCase):
         GPIO.setwarnings(False)
         with open('/sys/class/gpio/export','wb') as f:
             f.write(str(LED_PIN_BCM).encode())
+        time.sleep(0.2)  # wait for udev to set permissions
         with open('/sys/class/gpio/gpio%s/direction'%LED_PIN_BCM,'wb') as f:
             f.write(b'out')
-        with open('/sys/class/gpio/gpio%s/value'%LED_PIN_BCM,'wb') as f:
-            f.write(b'1')
         with warnings.catch_warnings(record=True) as w:
             GPIO.setup(LED_PIN, GPIO.OUT)    # generate 'already in use' warning
             self.assertEqual(len(w),0)       # should be no warnings
@@ -280,10 +292,9 @@ class TestSetWarnings(unittest.TestCase):
         GPIO.setwarnings(True)
         with open('/sys/class/gpio/export','wb') as f:
             f.write(str(LED_PIN_BCM).encode())
+        time.sleep(0.2)  # wait for udev to set permissions
         with open('/sys/class/gpio/gpio%s/direction'%LED_PIN_BCM,'wb') as f:
             f.write(b'out')
-        with open('/sys/class/gpio/gpio%s/value'%LED_PIN_BCM,'wb') as f:
-            f.write(b'1')
         with warnings.catch_warnings(record=True) as w:
             GPIO.setup(LED_PIN, GPIO.OUT)    # generate 'already in use' warning
             self.assertEqual(w[0].category, RuntimeWarning)
@@ -605,6 +616,27 @@ class TestEdgeDetection(unittest.TestCase):
         GPIO.wait_for_edge(LOOP_IN, GPIO.RISING)
         GPIO.wait_for_edge(LOOP_IN, GPIO.FALLING)
 
+    def testWaitForEdgeTimeout(self):
+        def makehigh():
+            GPIO.output(LOOP_OUT, GPIO.HIGH)
+        def makelow():
+            GPIO.output(LOOP_OUT, GPIO.LOW)
+
+        with self.assertRaises(TypeError):
+            GPIO.wait_for_edge(LOOP_IN, GPIO.RISING, timeout="beer")
+
+        with self.assertRaises(ValueError):
+            GPIO.wait_for_edge(LOOP_IN, GPIO.RISING, timeout=-1234)
+
+        makelow()
+        chan = GPIO.wait_for_edge(LOOP_IN, GPIO.RISING, timeout=200)
+        self.assertEqual(chan, None)
+
+        t = Timer(0.1, makehigh)
+        t.start()
+        chan = GPIO.wait_for_edge(LOOP_IN, GPIO.RISING, timeout=200)
+        self.assertEqual(chan, LOOP_IN)
+
     def tearDown(self):
         GPIO.cleanup()
 
